/*
    IrrMeshBuild.cpp
    Helper functions for imb files.

    MUTATE Systems 2018 (https://mutate.jp)
*/

#include "IrrMeshBuild.h"
#include <stdio.h>

// Implementation of Archive_File

Archive_File::~Archive_File() {
    close();
}

bool Archive_File::open(const char* szFileName, bool bLoad) {
    close();
    m_fp = fopen(szFileName, bLoad ? "rb" : "wb");
    m_bLoading = bLoad;
    return m_fp != NULL;
}

bool Archive_File::close(void) {
    if(!m_fp) return false;
    fclose(m_fp);
    m_fp = NULL;
    return true;
}

size_t Archive_File::serialize(void* pv, size_t len) {
    if(!m_fp) return 0;
    return m_bLoading ? fread(pv, 1, len, m_fp) : fwrite(pv, 1, len, m_fp);
}

// Serialization methods of IMB elements

template<class T> bool serialize(T* val, BinaryArchive* arc) {
    arc->serialize(val, sizeof(T));
    return true;
}

template<class T> bool serialize(std::vector<T>* val, BinaryArchive* arc) {
    size_t len = val->size();
    serialize(&len, arc);
    if(arc->isLoading())
        val->resize(len);
    for(auto i = val->begin(); i != val->end(); i ++)
        serialize(&(*i), arc);
    return true;
}

static bool serialize(std::string* pstr, BinaryArchive* arc) {
    size_t len = 0;
    if(arc->isLoading()) {
        arc->serialize(&len, sizeof(len));
        char* buf = new char[len];
        arc->serialize(buf, len);
        pstr->assign(buf);
        delete[] buf;
    }
    else {
        len = pstr->length() + 1;
        serialize(&len, arc);
        arc->serialize((void*)(pstr->c_str()), len);
    }
    return true;
}

static bool serialize(SIrrMeshBuild::Node* o, BinaryArchive* arc) {
    serialize(&o->name, arc);
    serialize(&o->nameParent, arc);
    serialize(&o->mtxLocal, arc);
    serialize(&o->meshes, arc);
    return true;
}

static bool serialize(SIrrMeshBuild::MeshBuffer::Bone* o, BinaryArchive* arc) {
    serialize(&o->name, arc);
    serialize(&o->mtxOffset, arc);
    serialize(&o->weights, arc);
    return true;
}

static bool serialize(SIrrMeshBuild::MeshBuffer* o, BinaryArchive* arc) {
    serialize(&o->scDiffuse, arc);
    serialize(&o->scAmbient, arc);
    serialize(&o->scEmissive, arc);
    serialize(&o->scSpecular, arc);
    serialize(&o->scTransparent, arc);
    serialize(&o->fShininess, arc);
    serialize(&o->matType, arc);
    serialize(&o->txDiffuse, arc);
    serialize(&o->txNormal, arc);
    serialize(&o->matFlags, arc);

    serialize(&o->verts, arc);
    serialize(&o->Tangents, arc);
    serialize(&o->Binormals, arc);
    serialize(&o->TCoords2, arc);
    serialize(&o->indices, arc);
    serialize(&o->bHasNormal, arc);
    serialize(&o->bones, arc);
    return true;
}

static bool serialize(SIrrMeshBuild::Animation* o, BinaryArchive* arc) {
    serialize(&o->speed, arc);
    serialize(&o->channels, arc);
    return true;
}

static bool serialize(SIrrMeshBuild::Animation::Channel* o, BinaryArchive* arc) {
    serialize(&o->nodeName, arc);
    serialize(&o->posKeys, arc);
    serialize(&o->rotationKeys, arc);
    serialize(&o->scaleKeys, arc);
    return true;
}

const unsigned ID_IMB_TAG = 0x31424d49; // IMB1

bool serialize(SIrrMeshBuild* o, BinaryArchive* arc) {
    unsigned tag = ID_IMB_TAG;
    serialize(&tag, arc);
    if(tag != ID_IMB_TAG) return false;

    serialize(&o->nodes, arc);
    serialize(&o->meshbufs, arc);
    serialize(&o->animations, arc);
    return true;
}
