/*
    IrrMeshBuild.h
    Mesh data structure for irrlicht engline.

    MUTATE Systems 2018 (https://mutate.jp)
*/

#pragma once
#include <vector>
#include <string>

#include <irrlicht.h>

struct SIrrMeshBuild {

    struct Node {
        std::string name, nameParent;
        irr::core::matrix4 mtxLocal;
        std::vector<irr::u32> meshes;
    };
    std::vector<Node> nodes;

    struct MeshBuffer {
        irr::video::SColor scDiffuse, scAmbient, 
            scEmissive, scSpecular, scTransparent;
        irr::f32 fShininess;
        irr::video::E_MATERIAL_TYPE matType;
        std::string txDiffuse, txNormal;

        struct FlagPair {
            irr::video::E_MATERIAL_FLAG emf;
            union {
                unsigned dwVal; bool bVal;
            };
        };
        std::vector<FlagPair> matFlags;

        std::vector<irr::video::S3DVertex> verts;
        std::vector<irr::core::vector3df> Tangents;
        std::vector<irr::core::vector3df> Binormals;
        std::vector<irr::core::vector2df> TCoords2;
        std::vector<irr::u32> indices; // must be counverted to u16
        bool bHasNormal;

        struct Bone {
            std::string name;
            irr::core::matrix4 mtxOffset;
            struct Weight {
                float weight;
                unsigned idVertex;
            };
            std::vector<Weight> weights;
        };
        std::vector<Bone> bones;
    };
    std::vector<MeshBuffer> meshbufs;

    struct Animation {
        irr::f32 speed;
        struct Channel {
            std::string nodeName;
            struct Key3d {
                irr::f32 frame;
                irr::core::vector3df val;
            };
            struct RotationKey {
                irr::f32 frame;
                irr::core::quaternion val;
            };
            std::vector<Key3d> posKeys;
            std::vector<RotationKey> rotationKeys;
            std::vector<Key3d> scaleKeys;
        };
        std::vector<Channel> channels;
    };
    std::vector<Animation> animations;
};

class BinaryArchive {
protected:
    bool m_bLoading;
    BinaryArchive(void) {m_bLoading = true; }
public:
    virtual size_t serialize(void* pv, size_t len) = 0;
    bool isLoading(void) { return m_bLoading; }
};

// File archive class

class Archive_File : public BinaryArchive {
    FILE* m_fp;
protected:
    virtual size_t serialize(void* pv, size_t len) override;
public:
    Archive_File(void) : m_fp(NULL) {}
    ~Archive_File();
    bool open(const char* szFileName, bool bLoad);
    bool close(void);
};

bool serialize(SIrrMeshBuild* o, BinaryArchive* arc);
