/*
    imb.cpp
    Command line imb converter.

    MUTATE Systems 2018 (https://mutate.jp)
*/

#include <stdio.h>
#include <io.h>
#include <stdio.h>
#include <string.h>
#include <windows.h>
#include "AssimpRead.h"
#include "IrrMeshBuild.h"
#include "common/tstring.h"

using std::string;

static std::string appendFilePath(const char* szDir, const char* szFileName) {
    if(!szDir || !*szDir) return std::string(szFileName);
    size_t len = strLen(szDir);
    char last = szDir[len - 1];
    return (last == '\\' || last == '/') ?
        std::string(szDir) + szFileName :
        std::string(szDir) + "\\" + szFileName ;
}

typedef void (*pfFileSpecCallback)(const char*, void*);

int parseFileSpec(const char* szFilter, pfFileSpecCallback pfunc, void* pParam) {
    _finddata_t fileinfo;
    long h = _findfirst(szFilter, &fileinfo);

    if(h == -1) {
        printf("No file found : %s\n", szFilter);
        return 0;
    }
    
    char szDir[_MAX_PATH];
    *szDir = 0;
    const char* p = NULL;
    if(p = strrchr(szFilter, '\\')) {
        int n = p - szFilter + 1;
        strncpy(szDir, szFilter, n);
        szDir[n] = 0;
        printf("Dir = %s\n", szDir);
    }

    int numFile = 0;
    int r = 0;
    do {
        char fnfull[_MAX_PATH];
        strcpy(fnfull, szDir);
        strcat(fnfull, fileinfo.name);
        pfunc(fnfull, pParam);
        numFile ++;
        r=_findnext(h, &fileinfo);
    } while(r==0);

    return numFile;
}

struct SConvertFileOptions {
    bool bSkeleton;
    std::set<std::string>* txnames;
    std::string strOutDir;
};

void convertFile(const char* szFileName, void* pParam) {
    auto cfo = (SConvertFileOptions*)pParam;
    printf("convertFile %s\n", szFileName);
    SIrrMeshBuild imb;
    if(!readMeshFile(&imb, szFileName, cfo->txnames)) {
        printf("Cannot recognize %s\n", szFileName);
        return;
    }
    auto pdot = strrchr(szFileName, '.');

    auto strFileNameBody = pdot ? string(szFileName, pdot - szFileName) : string(szFileName);
    auto strSave = appendFilePath(cfo->strOutDir.c_str(), strFileNameBody.c_str()).append(".imb");

    Archive_File file;

    if(!file.open(strSave.c_str(), false)) {
        printf("Cannot write to %s\n", strSave.c_str());
        return;
    }

    if(cfo->bSkeleton) {
        for(auto i = imb.meshbufs.begin(); i != imb.meshbufs.end(); i ++) {
            (*i).verts.clear();
            (*i).Tangents.clear();
            (*i).Binormals.clear();
            (*i).TCoords2.clear();
            (*i).indices.clear();
            (*i).bHasNormal = false;
            for(auto j = (*i).bones.begin(); j != (*i).bones.end(); j ++) {
                (*j).weights.clear();
            }
        }
    }

    serialize(&imb, &file);
}

int main(int argc, char* argv[]) {
    SConvertFileOptions cfo = {0};
    std::set<std::string> txnames;
    cfo.txnames = &txnames;

    const char* fileSpec = NULL;
    const char* szImbDir = NULL;
    const char* szTexInDir = NULL;
    const char* szTexOutDir = NULL;
    bool bError = false;
    for(int i = 1; i < argc; i ++) {
        const char* sz = argv[i];
        if(*sz == '/') {
            sz ++;
            if(!strcmp(sz, "s")) {
                cfo.bSkeleton = true;
            }
            else if(strBegin(sz, "o:")) {
                szImbDir = sz + 2;
            }
            else if(strBegin(sz, "ti:")) {
                szTexInDir = sz + 3;
            }
            else if(strBegin(sz, "to:")) {
                szTexOutDir = sz + 3;
            }
            else {
                printf("Unknown option: %s\n", sz);
                bError = true;
            }
        }
        else {
            if(fileSpec)
                bError = true;
            else
                fileSpec = sz;
        }
    }

    if(bError || !fileSpec) {
        puts("Usage: imb <options> xxx.fbx");
        return -1;
    }
    printf("Output directory : %s\n",  szImbDir ? szImbDir : "(default)");

    cfo.strOutDir.assign(szImbDir ? szImbDir : "");
    parseFileSpec(fileSpec, convertFile, &cfo);

    if(szTexOutDir && !cfo.bSkeleton) {
        printf("Texture input directory : %s\n", szTexInDir ? szTexInDir : "(Same as mesh folder)");
        printf("Texture output directory : %s\n", szTexOutDir);

        for(auto i = txnames.begin(); i != txnames.end(); i ++) {
            auto fromName =  appendFilePath(szTexInDir, (*i).c_str());
            auto toName = appendFilePath(szTexOutDir, (*i).c_str());
            auto bResult = ::CopyFileA(fromName.c_str(), toName.c_str(), FALSE);
            printf("Copy %s => %s : %s\n", fromName.c_str(), toName.c_str(), bResult ? "OK" : "Failed");
//            printf("Texture: %s\n", (*i).c_str());
        }
    }
}
